@@ -16,7 +16,7 @@ function(name, ..., existing = TRUE)
      # Will want to allow for class IDs to be specified here.
  name <-  as.character(name)
  if(existing) {
-      # force
+      # force 
    ans = getCOMInstance(name, force = TRUE, silent = TRUE)
    if(!is.null(ans) && !is.character(ans))
      return(ans)
@@ -25,7 +25,7 @@ function(name, ..., existing = TRUE)
  ans <- .Call("R_create", name, PACKAGE = "RDCOMClient")
  if(is.character(ans))
      stop(ans)
-
+ 
  ans
 }
 
@@ -58,7 +58,7 @@ function(guid, force = TRUE, silent = FALSE)
 
    status = looksLikeUUID(guid)
    if(status == 1) {
-        # substring(guid, 1, 1) != "{" && substring(guid, nchar(guid)) != "}"
+        # substring(guid, 1, 1) != "{" && substring(guid, nchar(guid)) != "}" 
      warning("guid must have form '{.....}', i.e. the curly braces around the id.")
      guid = paste("{", guid, "}", sep = "")
    }
@@ -69,7 +69,7 @@ function(guid, force = TRUE, silent = FALSE)
     if(!force) {
       if(silent)
         return(ans)
-      els
+      else
         stop(ans)
     } else {
        if(!silent)
@@ -101,24 +101,24 @@ setMethod("$<-", signature("COMIDispatch"), function(x, name, value) {
 # If we uncomment the following, we would have asymmetry
 #  in that x$foo <- 1 would work but x$foo would not.
 #	     .Call("R_setProperty", x, as.character(name), list(value), integer(0))
-#	      x
+#	      x 
 
           })
 
-setMethod("$", signature("COMIDispatch"),
+setMethod("$", signature("COMIDispatch"), 
 	    function(x, name){
 	       function(...) {
 	         .COM(x, name, ...)
 	       }
 	    })
 
-setMethod("[[", c("COMIDispatch", "numeric"),
+setMethod("[[", c("COMIDispatch", "numeric"), 
 	      function(x, i, j, ...) {
 # if i is numeric, can check if there is an Item() method.
 	       .COM(x, "Item", i)
 	      })
 
-setMethod("[[", "COMIDispatch",
+setMethod("[[", "COMIDispatch", 
 	      function(x, i, j, ...) {
 # if i is numeric, can check if there is an Item() method.
 	       .Call("R_getProperty", x, as.character(i), NULL, integer(0), PACKAGE = "RDCOMClient")
@@ -134,25 +134,37 @@ setMethod("[[<-", c("COMIDispatch", "character", "character"),
 
 setMethod("[[<-", c("COMIDispatch", "character", "missing"),
 	    function(x, i, j, ..., value) {
-if(length(i) > 1) {
-  tmp = x
-  for(id in i[-length(i)])
-     tmp = tmp[[ id ]]
-  .Call("R_setProperty", tmp, as.character(i[length(i)]), list(value), integer(0), PACKAGE = "RDCOMClient")
-} else
-              .Call("R_setProperty", x, as.character(i), list(value), integer(0), PACKAGE = "RDCOMClient")
-	      x
+
+
+	    if(length(i) > 1) {
+	      tmp = x
+	        for(id in i[-length(i)])
+		     tmp = tmp[[ id ]]
+		       .Call("R_setProperty", tmp, as.character(i[length(i)]), list(value), integer(0), PACKAGE = "RDCOMClient")
+	    } else {
+
+	    id = .Call("R_lookupPropName", x, as.character(i), PACKAGE = "RDCOMClient")
+	    if(is.na(id)) {
+	      
+	       stop(structure(list(message = paste("no property ", i, " in this COM object"), call = NULL),
+	             class = c("InvalidPropertyName", "DCOMError", "error", "condition")))
+            }
+            if(!.Call("R_isReadOnly", x, id, PACKAGE = "RDCOMClient"))
+               .Call("R_setProperty", x, as.character(i), list(value), id, PACKAGE = "RDCOMClient")
+       }
+	      x 
   	    })
 
 setMethod("[[<-", c("COMIDispatch", "numeric"),
 	    function(x, i, j, ..., value) {
-	      x
+	      warning("x[[ number ]] <- value doesn't change anything in a COMIDispatch object")
+	      x 
   	    })
 
 .COM <-
  # Allows one to control the type of dispatch used in the COM Invoke() call.
  # Useful for getting properties and methods.
-function(obj, name,  ..., .dispatch = as.integer(3), .return = TRUE, .ids=numeric(0),
+function(obj, name,  ..., .dispatch = 3L, .return = TRUE, .ids=numeric(0),
            .suppliedArgs)
 {
  # if(!missing(.ids)) { }
@@ -160,7 +172,7 @@ function(obj, name,  ..., .dispatch = as.integer(3), .return = TRUE, .ids=numeri
  .args = list(...)
  if(!missing(.suppliedArgs))
       .args =  .args[!is.na(.suppliedArgs)]
- val = .Call("R_Invoke", obj, as.character(name), .args,
+ val = .Call("R_Invoke", obj, as.character(name), .args, 
 	  	as.integer(.dispatch), as.logical(.return), as.numeric(.ids), PACKAGE = "RDCOMClient")
 
  val
@@ -189,7 +201,7 @@ function(ref, className)
 }
 
 
-isValidCOMObject =
+isValidCOMObject = 
 function(obj)
 {
   if(!is(obj, "COMIDispatch"))
@@ -198,6 +210,6 @@ function(obj)
   if(!.Call("R_isValidCOMObject", obj, PACKAGE = "RDCOMClient"))
     return(FALSE)
 
-   # Now run the .COM(obj, "isValidObject")
+   # Now run the .COM(obj, "isValidObject") 
   TRUE
 }
