@@ -5,16 +5,46 @@
 
 #include <tchar.h>
 
+//extern "C"
+int RDCOM_WriteErrors = 0;
+static FILE *f = NULL;
+
+extern "C"
+SEXP
+RDCOM_setWriteError(SEXP value)
+{
+  int tmp = RDCOM_WriteErrors;  
+  if(TYPEOF(value) == STRSXP) {
+    if(f)
+      fclose(f);
+    
+    f = fopen(CHAR(STRING_ELT(value, 0)), "a");
+    RDCOM_WriteErrors = 1;
+    
+  } else {
+    RDCOM_WriteErrors = asLogical(value);
+    if(f)
+      fflush(f);
+  }
+
+    return(ScalarLogical(tmp));
+}
+
+extern "C"
+SEXP
+RDCOM_getWriteError(SEXP value)
+{
+    return(ScalarLogical(RDCOM_WriteErrors));
+}
 
 FILE *
 getErrorFILE()
 {
-  static FILE *f = NULL;
   if(!f) {
-    f = fopen("C:\\RDCOM.err", "a");
-    if(!f) {
-      f = fopen("C:\\RDCOM_server.err", "a");
-    }
+    //    f = fopen("%TMP%\\RDCOM.err", "a");
+    //    if(!f) {
+    //      f = fopen("C:\\RDCOM_server.err", "a");
+    //    }
   }
   return(f);
 }
@@ -344,7 +374,10 @@ SEXP R_createCOMErrorCodes();
 
 
 
+#ifndef _countof
 #define _countof(array) (sizeof(array)/sizeof(array[0]))
+#endif
+
 void GetScodeString(HRESULT hr, LPTSTR buf, int bufSize)
 {
 	// first ask the OS to give it to us..
@@ -368,7 +401,7 @@ void GetScodeString(HRESULT hr, LPTSTR buf, int bufSize)
 		}
 	}
 	// not found - make one up
-	sprintf(buf, ("OLE error 0x%08x"), hr);
+	sprintf(buf, ("OLE error 0x%ld"), hr);
 }
 
 
@@ -406,7 +439,8 @@ checkErrorInfo(IUnknown *obj, HRESULT status, SEXP *serr)
   HRESULT hr;
   ISupportErrorInfo *info;
 
-  fprintf(stderr, "<checkErrorInfo> %X \n", status);
+  // XXX DON'T print like this
+  fprintf(stderr, "<checkErrorInfo> %ld \n", status);
 
   if(serr) 
     *serr = NULL;
