@@ -362,7 +362,7 @@ SEXP
 R_convertDCOMObjectToR(VARIANT *var)
 {
   SEXP ans = R_NilValue;
-  HRESULT hr;
+  HRESULT hr = S_OK;
 
   VARTYPE type = V_VT(var);
 
@@ -446,7 +446,7 @@ R_convertDCOMObjectToR(VARIANT *var)
     case VT_I2:
     case VT_I4:
     case VT_INT:
-      hr = VariantChangeType(var, var, 0, VT_I4);
+      hr =  VariantChangeType(var, var, 0, VT_I4);
       ans = R_scalarInteger(V_I4(var));
       break;
 
@@ -503,11 +503,21 @@ R_convertDCOMObjectToR(VARIANT *var)
   case VT_CARRAY:
   case VT_USERDEFINED:
   default:
+    //XXXX 
     fprintf(stderr, "Unhandled conversion type %d\n", V_VT(var));fflush(stderr);
     //XXX this consumes the variant. So the variant clearance in Invoke() does it again!
     ans = createRVariantObject(var, V_VT(var));
   }
 
+  if(hr != S_OK) {
+    char errBuf[1000];
+    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, hr, // GetLastError(), 
+		  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
+		  errBuf, sizeof(errBuf)/sizeof(errBuf[0]), NULL);
+    PROBLEM "problem in conversion (%ld) %s", hr, errBuf
+      ERROR;
+  }
+
 #if defined(RDCOM_VERBOSE) && RDCOM_VERBOSE
   errorLog("Finished convertDCOMObjectToR\n");
 #endif
@@ -561,7 +571,7 @@ createRDCOMArray(SEXP obj, VARIANT *var)
 
   HRESULT hr = SafeArrayAccessData(arr, (void**) &data);
   if(hr != S_OK) {
-    std::cerr <<"Problems accessing data" << std::endl;
+    errorLog("Problems accessing data");
     SafeArrayDestroy(arr);
     return(NULL);
   }
@@ -590,7 +600,9 @@ createRDCOMArray(SEXP obj, VARIANT *var)
       break;
 
     default:
-      std::cerr <<"Array case not handled yet for R type " << TYPEOF(obj) << std::endl;
+      PROBLEM "Array case not handled yet for R type %d", TYPEOF(obj)
+      WARN;
+      
     break;
   }
 
@@ -855,6 +867,7 @@ R_create2DArray(SEXP obj)
   return(ans);
 }
 
+extern "C"
 SEXP
 R_createVariant(SEXP type)
 {
